// FileElement.h: Schnittstelle fr die Klasse CFileElement.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FILEELEMENT_H__131617E2_F565_45A8_8EB7_E5615E8BB849__INCLUDED_)
#define AFX_FILEELEMENT_H__131617E2_F565_45A8_8EB7_E5615E8BB849__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vector>
using namespace std;

class CFileElement  
{
public:
	CFileElement();
	CFileElement(CString strName, _int64 nSize, CTime tCreated, 
							CTime tLastModified, bool bIsDir);

	virtual ~CFileElement();

	// delete this node and all subnodes
	void clear();

	// add a subnode to this node
	bool add(CString strFilename, CString strRoot, CTime tCreation, 
			CTime tLastModified, _int64 nSize, bool bIsDir);

	// rempve a subnode
	bool remove(CString strFilename);

	// get subnodes
	CFileElement* get(CString strFilename);							// by Filename
	CFileElement* get(float x1, float y1, float x2, float y2);		// by Position

	int getMiddleChildID(int nMin, int nMax);
	int getLargestChildID(int nMin, int nMax);
	int getDepth(CFileElement *fe, int depth);

	// get all nodes in a directory
	std::vector<CFileElement *> *getDirectory();

	static CFileElement* findElementAt(CFileElement *fe, float x, float y, 
			int nDepth, int nCurrDepth);

	// randomize (size)
	void randomizeSize(CFileElement *fe);
	_int64 recalculateSize(CFileElement *fe);

	// debugPrint
	void printTree(CFileElement *fe, int level = 0);

	// getters
	inline CString getName()			{	return m_strName;					}
	inline CTime getCreated()			{	return m_tCreated;					}
	inline CTime getLastModifed()		{	return m_tLastModified;				}
	inline _int64 getSize()				{	return m_nSize;						}
	inline bool IsDirectory()			{	return m_bIsDirectory;				}
	inline float* getPoints()			{	return m_fPoints;					}
	inline float* getColor()			{	return m_fColor;					}

	// min/max - getters
	inline _int64 getMinSize()			{	return m_nMinSize;					}
	inline _int64 getMaxSize()			{	return m_nMaxSize;					}
	inline CTime getMinCreated()		{	return m_tMinCreated.GetTime();		}
	inline CTime getMaxCreated()		{	return m_tMaxCreated.GetTime();		}
	inline CTime getMinModified()		{	return m_tMinModified.GetTime();	}
	inline CTime getMaxModified()		{	return m_tMaxModified.GetTime();	}
	inline int getDepth()				{	return m_nDepth;					}


	// setters
	void setRoot(CString strName);
	void setName(CString strName);
	void setCreated(CTime tTime);
	void setLastModified(CTime tTime);
	void setSize(int nSize);
	void setIsDirectory(bool bDirectory);
	void setPoints(float x1, float y1, float x2, float y2);
	void setColor(float cRed, float cGreen, float cBlue);

private:
	CString	m_strName;					// filename
	CTime	m_tCreated;					// created date
	CTime	m_tLastModified;			// last modified date
	_int64	m_nSize;					// filesize in bytes
	bool	m_bIsDirectory;				// indicates whether this is an directory
	float	m_fPoints[4];				// left-lower & right-upper Point of the Rectangle
	float	m_fColor[3];				// color of this rectangle

	std::vector<CFileElement *> next;	// vector of CFileElement (=nodes in this directory)

	void clear(CFileElement *fe, int nLevel = 0);

	int m_nDepth;

	// min-max values
	_int64 m_nMinSize;
	_int64 m_nMaxSize;
	CTime m_tMinCreated;
	CTime m_tMaxCreated;
	CTime m_tMinModified;
	CTime m_tMaxModified;
	
};

#endif // !defined(AFX_FILEELEMENT_H__131617E2_F565_45A8_8EB7_E5615E8BB849__INCLUDED_)
