// PosGL.cpp: Implementierungsdatei
//

#include "stdafx.h"
#include "OTM.h"
#include "OpenGL.h"
#include "OTMDlg.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// COpenGL

COpenGL::COpenGL(CWnd *pParent)
{	
	m_pParent = pParent;

	CRect r;
	m_pParent->GetClientRect(&r);
	r.right = r.right - 220;


	Create(NULL, NULL, WS_CHILD|WS_CLIPSIBLINGS|WS_CLIPCHILDREN|WS_VISIBLE,
		   r, m_pParent, 541);


	m_hGLContext = NULL;

	HWND hWnd = GetSafeHwnd();
	HDC hDC = ::GetDC(hWnd);

	if(MySetPixelFormat(hDC)==FALSE)
		AfxMessageBox("MySetPixelFormat(hDC)==FALSE");
	
	m_hGLContext = wglCreateContext(hDC);

	if(m_hGLContext==NULL)
		AfxMessageBox("m_hGLContext==NULL");

	if(wglMakeCurrent(hDC,m_hGLContext)==FALSE)
		AfxMessageBox("wglMakeCurrent(hDC,m_hGLContext)==FALSE");

	m_nTexWidth	= 512;
	m_nTexHeight = 512;
	m_nTexDepth	= 4;
	m_bTexValid = false;
	m_nCurrDepth = 0;
	
	m_nTexID = createEmptyTexture(m_nTexWidth, m_nTexHeight, m_nTexDepth);
	GLinit();

	SetTimer(0,10,NULL);

	// detach context
	if(wglGetCurrentContext() != NULL)
		wglMakeCurrent(NULL,NULL);

	::ReleaseDC (m_hWnd, hDC) ; 

	renderer = new CRenderer();
	Resize();
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

COpenGL::~COpenGL() 
{
	if(wglGetCurrentContext() != NULL)
		wglMakeCurrent(NULL,NULL);
	
	if(m_hGLContext != NULL)
	{
		wglDeleteContext(m_hGLContext);
		m_hGLContext = NULL;
	}

	delete renderer;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

BEGIN_MESSAGE_MAP(COpenGL, CWnd)
	//{{AFX_MSG_MAP(COpenGL)
	ON_WM_PAINT()
	ON_WM_TIMER()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

void COpenGL::Resize()
{
	CRect r;
	m_pParent->GetClientRect(&r);
	r.right = r.right - 220;
	
	// update window size
	SetWindowPos(NULL, r.top, r.left, r.Width(), r.Height(), SWP_NOZORDER);

	GLresize(r.Width(), r.Height());
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

void COpenGL::Randomize()
{
	renderer->getFileElement()->randomizeSize(renderer->getFileElement());
	renderer->getFileElement()->recalculateSize(renderer->getFileElement());
	render();
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

GLuint COpenGL::createEmptyTexture(short width, short height, short depth)
{
	GLuint txtnumber = 1;	// Texture ID

	unsigned int* data;		// Stored Data

	// Create Storage Space For Texture Data
	data = (unsigned int*)new GLuint[((width * height)* depth * sizeof(unsigned int))];

	// Clear Storage Memory
	ZeroMemory(data,((width * height)* depth * sizeof(unsigned int)));

	glGenTextures(1, &txtnumber);				// Create 1 Texture
	glBindTexture(GL_TEXTURE_2D, txtnumber);	// Bind The Texture

	// Build Texture Using Information In data
	glTexImage2D(GL_TEXTURE_2D, 0, depth, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);//_MIPMAP_NEAREST );
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);//_MIPMAP_NEAREST );

	delete [] data;		// Release data

	return txtnumber;
}

/////////////////////////////////////////////////////////////////////////////

void COpenGL::renderSceneToTexture(short width, short height)
{
	GLint *ask = new GLint[4];
	glGetIntegerv(GL_VIEWPORT,ask);

	glViewport(0,0,width,height);	// Set Our Viewport (Match Texture Size)

	renderScene();

	glBindTexture(GL_TEXTURE_2D, m_nTexID);					// Bind To The Blur Texture

	// Copy Our ViewPort To The Blur Texture (From 0,0 To width,height... No Border)
	glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 0, 0, width, height, 0);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);						// Set The Clear Color To Medium Blue
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);			// Clear The Screen And Depth Buffer

	glViewport(ask[0], ask[1], ask[2], ask[3]);	
	delete [] ask;
}

/////////////////////////////////////////////////////////////////////////////

void COpenGL::renderScene()
{
	CFileElement *fe = renderer->getFileElement();

	if (fe == NULL)
	{
		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer
		glLoadIdentity();		
	}
	else
	{
		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer
		glLoadIdentity();

		drawFileElement(fe, 0);
	}
}

/////////////////////////////////////////////////////////////////////////////

void COpenGL::drawFileElement(CFileElement *fe, int nDepth)
{
	if (nDepth > m_nCurrDepth) return;

	vector<CFileElement *> *dir = fe->getDirectory();

	if ((dir->size() == 0) || (nDepth == m_nCurrDepth))
	{
		float *fPoints	= fe->getPoints();
		float *fCol		= fe->getColor();
		
		// draw the quad
		glBegin(GL_QUADS);
			glColor3f(fCol[0], fCol[1], fCol[2]);
			glVertex2f(fPoints[0], fPoints[1]);
			glVertex2f(fPoints[2], fPoints[1]);
			glVertex2f(fPoints[2], fPoints[3]);
			glVertex2f(fPoints[0], fPoints[3]);
		glEnd();
	}
	else
	{
		for (int i = 0; i < dir->size(); i++)
		{
			drawFileElement((CFileElement *) dir->at(i), nDepth + 1);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

void COpenGL::GLinit()
{
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	// clear color buffer
	glClearDepth(1.0f);						// clear depth buffer

	glEnable(GL_DEPTH_TEST);				// enable depth testing 
	glEnable(GL_POLYGON_SMOOTH );
	glDisable(GL_CULL_FACE);
	glEnable(GL_LINE_SMOOTH);

	glShadeModel(GL_SMOOTH);				// enable smooth shading
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

BOOL COpenGL::MySetPixelFormat(HDC hDC)
{
	int m_GLPixelIndex = 0;
	PIXELFORMATDESCRIPTOR pixelDesc;

	pixelDesc.nSize = sizeof(PIXELFORMATDESCRIPTOR);
	pixelDesc.nVersion = 1;

	pixelDesc.dwFlags = PFD_DRAW_TO_WINDOW | 
						PFD_SUPPORT_OPENGL |
						PFD_DOUBLEBUFFER |
						PFD_STEREO_DONTCARE;

	pixelDesc.iPixelType = PFD_TYPE_RGBA;
	pixelDesc.cColorBits = 32;
	pixelDesc.cRedBits = 8;
	pixelDesc.cRedShift = 16;
	pixelDesc.cGreenBits = 8;
	pixelDesc.cGreenShift = 8;
	pixelDesc.cBlueBits = 8;
	pixelDesc.cBlueShift = 0;
	pixelDesc.cAlphaBits = 0;
	pixelDesc.cAlphaShift = 0;
	pixelDesc.cAccumBits = 64;
	pixelDesc.cAccumRedBits = 16;
	pixelDesc.cAccumGreenBits = 16;
	pixelDesc.cAccumBlueBits = 16;
	pixelDesc.cAccumAlphaBits = 0;
	pixelDesc.cDepthBits = 32;
	pixelDesc.cStencilBits = 8;
	pixelDesc.cAuxBuffers = 0;
	pixelDesc.iLayerType = PFD_MAIN_PLANE;
	pixelDesc.bReserved = 0;
	pixelDesc.dwLayerMask = 0;
	pixelDesc.dwVisibleMask = 0;
	pixelDesc.dwDamageMask = 0;

	m_GLPixelIndex = ChoosePixelFormat(hDC,&pixelDesc);
	if(m_GLPixelIndex==0) // Choose default
	{
		m_GLPixelIndex = 1;
		if(!DescribePixelFormat(hDC,m_GLPixelIndex,sizeof(PIXELFORMATDESCRIPTOR),&pixelDesc))
			return FALSE;
	}

	if(!SetPixelFormat(hDC,m_GLPixelIndex,&pixelDesc))
		return FALSE;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

void COpenGL::GLresize(GLsizei w, GLsizei h)
{
	HWND hWnd = GetSafeHwnd();
	HDC hDC = ::GetDC(hWnd);

	// try to make it the thread's current rendering context 
	if(wglMakeCurrent(hDC, m_hGLContext))
	{
		if (h == 0) h = 1;						// no divide by zero

		glViewport(0,0,w,h);					// reset the current viewport

		glMatrixMode(GL_PROJECTION);			// Select The Projection Matrix
		glLoadIdentity();						// Reset The Projection Matrix

		glOrtho(0.0, 1.0, 0.0, 1.0, -1.0, 1.0);

		glMatrixMode(GL_MODELVIEW);				// Select The Modelview Matrix
		glLoadIdentity();

		m_bTexValid = false;
	}

	// detach the current rendering context from the thread 
	wglMakeCurrent(NULL, NULL);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


void COpenGL::ViewOrtho(GLsizei w, GLsizei h)					// Set Up An Ortho View
{
	glMatrixMode(GL_PROJECTION);								// Select Projection
	glPushMatrix();												// Push The Matrix
	glLoadIdentity();											// Reset The Matrix
	glOrtho( 0, w, h , 0, -1, 1 );								// Select Ortho Mode (640x480)
	glMatrixMode(GL_MODELVIEW);									// Select Modelview Matrix
	glPushMatrix();												// Push The Matrix
	glLoadIdentity();											// Reset The Matrix
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

void COpenGL::ViewPerspective()									// Set Up A Perspective View
{
	glMatrixMode( GL_PROJECTION );								// Select Projection
	glPopMatrix();												// Pop The Matrix
	glMatrixMode( GL_MODELVIEW );								// Select Modelview
	glPopMatrix();												// Pop The Matrix
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

void COpenGL::OnPaint() 
{
	if (IsIconic() != 0)
	{
		// nothing to do
	}
	else
	{
		CPaintDC dc(this); // device context for painting
		HWND hWnd = GetSafeHwnd();
		HDC hDC = ::GetDC(hWnd);

		// try to make it the thread's current rendering context 
		if(wglMakeCurrent(hDC, m_hGLContext))
		{
			m_bTexValid = m_bTexValid && !renderer->IsDirty();
			// check if this texture is still valid
			if (!m_bTexValid)
			{
				// delete the old texture and create a new one
				glDeleteTextures(1, &m_nTexID);
				m_nTexID = createEmptyTexture(m_nTexWidth, m_nTexHeight, m_nTexDepth);
				renderSceneToTexture(m_nTexWidth, m_nTexHeight);

				// reset decision parameters
				renderer->ClearDirtyBit();
				m_bTexValid	= true;
			}

			// just display the texture
			glEnable(GL_TEXTURE_2D);				// Enable 2D Texture Mapping
			glBindTexture(GL_TEXTURE_2D, m_nTexID);	// Bind  Texture
			//glScalef(scale, scale, scale);

			glBegin(GL_QUADS);
				glColor3ub(255, 255, 255);
				glTexCoord2f(0, 0);	
				glVertex2f(0.0f, 0.0f);
				glTexCoord2f(1, 0);	
				glVertex2f(1.0f, 0.0f);
				glTexCoord2f(1, 1);	
				glVertex2f(1.0f, 1.0f);
				glTexCoord2f(0, 1);	
				glVertex2f(0.0f, 1.0);
			glEnd();

			glDisable(GL_TEXTURE_2D);				// Disable 2D Texture Mapping
			glBindTexture(GL_TEXTURE_2D,0);			// Unbind The Blur Texture

			SwapBuffers(hDC);
			CWnd::OnPaint();
		}

		// detach the current rendering context from the thread 
		wglMakeCurrent(NULL, NULL);

		::ReleaseDC(hWnd,hDC);
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

void COpenGL::OnTimer(UINT nIDEvent) 
{
	InvalidateRect(NULL,FALSE);	
	CWnd::OnTimer(nIDEvent);
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////EOF


void COpenGL::OnLButtonDown(UINT nFlags, CPoint point) 
{
	float x,y;
	CRect r;
	this->GetWindowRect(&r);

	x = (float) point.x / r.Width();
	y = 1.0f - (float) point.y / r.Height();
	
	CFileElement *fe = renderer->getFileElement();
	if (fe != NULL)
	{
		CFileElement *elem = CFileElement::findElementAt(fe, x, y, m_nCurrDepth, 0);
		if (elem != NULL)
		{
			((COTMDlg *) m_pParent)->m_panel->WriteFileElementInfo(elem);
			//TRACE("Found: %s, %d\n", elem->getName(), elem->getSize() / 1024);
		}
	}
	
	CWnd::OnLButtonDown(nFlags, point);
}