// Renderer.cpp: Implementierung der Klasse CRenderer.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "stdafx.h"
#include "Renderer.h"
#include  <math.h>

// or random coloring
#include <stdlib.h>
#include <time.h>



#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CRenderer::CRenderer()
{
	m_eType			= SLICE_AND_DICE;
	m_eColAttr		= SIZE;
	m_bDirty		= false;
	m_FileElement	= NULL;
	m_nMaxSize		= 0;

	m_dRange		= 0.0f;
	m_dMinValue		= 0.0f;

	m_dMinCol[0]	= 0.0f;
	m_dMinCol[1]	= 0.0f;
	m_dMinCol[2]	= 0.0f;

	m_dMaxCol[0]	= 1.0f;
	m_dMaxCol[1]	= 1.0f;
	m_dMaxCol[2]	= 1.0f;

	m_bHasRandomColor = false;

	CalcColorDiff();

	// init randomizer -> for color generation
	srand((unsigned) time(NULL));
}

//////////////////////////////////////////////////////////////////////

CRenderer::~CRenderer()
{

}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

void CRenderer::render()
{
	if (m_FileElement == NULL) return;

	int nMin, nMax;

	// calculate ranges
	switch(m_eColAttr)
	{
		case CREATION:
			m_dRange = (float) (m_FileElement->getMaxCreated().GetTime() - m_FileElement->getMinCreated().GetTime());
			m_dMinValue = (float) (m_FileElement->getMinCreated().GetTime());
			break;
		case MODIFIED:
			m_dRange = (float) (m_FileElement->getMaxModified().GetTime() - m_FileElement->getMinModified().GetTime());
			m_dMinValue = (float) (m_FileElement->getMinModified().GetTime());
			break;
		case RANDOM:
			// not needed
			break;
		case SIZE:	
			m_dRange = (float) (m_FileElement->getMaxSize() - m_FileElement->getMinSize());
			m_dMinValue = (float) (m_FileElement->getMinSize());
			break;
		default:
			m_dRange	= 1.0f;
			m_dMinValue = 0.0f;
			break;
	}

	// set Size to -> width: 1.0f, height: 1.0f;
	m_FileElement->setPoints(0.0f, 0.0f, 1.0f, 1.0f);
	m_nMaxSize = m_FileElement->getSize();

	switch (m_eType) 
	{
		case SLICE_AND_DICE:
			renderSliceAndDice(m_FileElement, true);
			break;
		case SQUARIFIED:
			// ???
			break;
		case PIVOT_BY_MIDDLE:
			nMin = 0;
			nMax = m_FileElement->getDirectory()->size() - 1;
			renderPivotByMiddle(m_FileElement, nMin, nMax, m_FileElement->getPoints());
			break;
		case PIVOT_BY_SIZE:
			nMin = 0;
			nMax = m_FileElement->getDirectory()->size() - 1;
			renderPivotBySize(m_FileElement, nMin, nMax, m_FileElement->getPoints());
			break;
		default: 
			break;
	}

	if (m_eColAttr == RANDOM) m_bHasRandomColor = true;
	else m_bHasRandomColor = false;

	m_bDirty = true;
}

//////////////////////////////////////////////////////////////////////

void CRenderer::setFileElement(CFileElement *fe)
{
	m_FileElement = fe;
}

//////////////////////////////////////////////////////////////////////

void CRenderer::setRenderType(RENDERTYPE rt)
{
	m_eType = rt;
}

//////////////////////////////////////////////////////////////////////

void CRenderer::setColorAttribute(COLATTRIBUTE att)
{
	m_eColAttr = att;
}

//////////////////////////////////////////////////////////////////////

void CRenderer::setMinColor(float dRed, float dGreen, float dBlue)
{
	m_dMinCol[0] = dRed;
	m_dMinCol[1] = dGreen;
	m_dMinCol[2] = dBlue;

	CalcColorDiff();
}

//////////////////////////////////////////////////////////////////////

void CRenderer::setMaxColor(float dRed, float dGreen, float dBlue)
{
	m_dMaxCol[0] = dRed;
	m_dMaxCol[1] = dGreen;
	m_dMaxCol[2] = dBlue;

	CalcColorDiff();
}

//////////////////////////////////////////////////////////////////////
//protected///////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

void CRenderer::setColor(CFileElement *fe)
{
	float dCol[3];
	float dFactor = 0.0f;

	switch(m_eColAttr)
	{
		case CREATION:
			dFactor = ((float) fe->getCreated().GetTime() - m_dMinValue) / m_dRange;
			dCol[0] = m_dMinCol[0] + m_dDiffCol[0] * dFactor;
			dCol[1] = m_dMinCol[1] + m_dDiffCol[1] * dFactor;
			dCol[2] = m_dMinCol[2] + m_dDiffCol[2] * dFactor;
			break;
		case MODIFIED:
			dFactor = ((float) fe->getLastModifed().GetTime() - m_dMinValue) / m_dRange;
			dCol[0] = m_dMinCol[0] + m_dDiffCol[0] * dFactor;
			dCol[1] = m_dMinCol[1] + m_dDiffCol[1] * dFactor;
			dCol[2] = m_dMinCol[2] + m_dDiffCol[2] * dFactor;
			break;
		case RANDOM:
			if (m_bHasRandomColor) return;
			dCol[0] = (float) rand() / RAND_MAX;
			dCol[1] = (float) rand() / RAND_MAX;
			dCol[2] = (float) rand() / RAND_MAX;
			break;
		case SIZE:	
			dFactor = ((float) fe->getSize() - m_dMinValue) / m_dRange;
			dCol[0] = m_dMinCol[0] + m_dDiffCol[0] * dFactor;
			dCol[1] = m_dMinCol[1] + m_dDiffCol[1] * dFactor;
			dCol[2] = m_dMinCol[2] + m_dDiffCol[2] * dFactor;
			break;
		default:
			dCol[0] = 1.0f;
			dCol[1] = 1.0f;
			dCol[2] = 1.0f;
			break;
	}

	fe->setColor(dCol[0], dCol[1], dCol[2]);
}

//////////////////////////////////////////////////////////////////////

void CRenderer::CalcColorDiff()
{
	m_dDiffCol[0] = (m_dMaxCol[0] - m_dMinCol[0]);
	m_dDiffCol[1] = (m_dMaxCol[1] - m_dMinCol[1]);
	m_dDiffCol[2] = (m_dMaxCol[2] - m_dMinCol[2]);
}

//////////////////////////////////////////////////////////////////////

void CRenderer::renderSliceAndDice(CFileElement *fe, bool bHorizontal)
{
	_int64 nSize = fe->getSize();						// filesize
	float *fPoints = fe->getPoints();					// points of this element
	vector<CFileElement *> *dir = fe->getDirectory();	// subnodes
	float fCurrPos = 0.0f;								// current Position (vertical or horizontal)

	// just prevention
	if (nSize == 0.0f) nSize = 1.0f;

	for (int i = 0; i < dir->size(); i++)
	{
		// fetch the actual element
		CFileElement *elem = (CFileElement *) dir->at(i);
		float fWidth;
		float fNewPoints[4];

		if (bHorizontal) 
		{
			fWidth = (fabs(fPoints[1] - fPoints[3]) * elem->getSize()) / nSize; 
			fNewPoints[0] = fPoints[0];
			fNewPoints[1] = fPoints[1] + fCurrPos;
			fNewPoints[2] = fPoints[2];
			fNewPoints[3] = fPoints[1] + fCurrPos + fWidth;

			fCurrPos += fWidth;
		}
		else
		{
			fWidth = (fabs(fPoints[0] - fPoints[2]) * elem->getSize()) / nSize;
			fNewPoints[0] = fPoints[0] + fCurrPos;
			fNewPoints[1] = fPoints[1];
			fNewPoints[2] = fPoints[0] + fCurrPos + fWidth;
			fNewPoints[3] = fPoints[3];

			fCurrPos += fWidth;
		}

		setColor(elem);
		elem->setPoints(fNewPoints[0], fNewPoints[1], fNewPoints[2], fNewPoints[3]);

		renderSliceAndDice(elem, !bHorizontal);
	}
}

//////////////////////////////////////////////////////////////////////

void CRenderer::renderPivotByMiddle(CFileElement *fe, int nMin, int nMax, float *dPoints)
{
	int nMiddleID = fe->getMiddleChildID(nMin, nMax);

	// if there are childs
	if (nMiddleID != -1)
	{
		float dWidth, dHeight;

		// sizes
		_int64 nSizeComplete = 0;
		_int64 nSizeRest = 0;
		_int64 nSizeL1 = 0;
		_int64 nSizeL2 = 0;
		_int64 nSizeL3 = 0;
		_int64 nSizePivot = 0;

		_int64 nPrefSizeL2 = 0;

		// list borders
		int nL1Min = 0;
		int nL1Max = 0;
		int nL2Min = 0;
		int nL2Max = 0;
		int nL3Min = 0;
		int nL3Max = 0;

		// pivot square
		float dPivotWidth  = 0.0f;
		float dPivotHeight = 0.0f;
		float dPivotArea   = 0.0f;

		float dCompleteArea = 0.0f;

		float dL1Width  = 0.0f;
		float dL1Height	= 0.0f;
		float dL2Width	= 0.0f;
		float dL2Height = 0.0f;
		float dL3Width  = 0.0f;
		float dL3Height = 0.0f;

		float dAreaL1 = 0.0f;
		float dAreaL2 = 0.0f;
		float dAreaL3 = 0.0f;

		bool bHorizontal = true;

		/*if (dPoints[2] < dPoints[0]) TRACE("error [X]!\n");
		else if (dPoints[3] < dPoints[1]) TRACE("error [Y]!\n");*/

		if ((dPoints[2] - dPoints[0]) >= (dPoints[3] - dPoints[1]))
		{
			dWidth	= fabs(dPoints[2] - dPoints[0]);
			dHeight = fabs(dPoints[3] - dPoints[1]);
		}
		else
		{
			dWidth	= fabs(dPoints[3] - dPoints[1]);
			dHeight = fabs(dPoints[2] - dPoints[0]);
			bHorizontal = false;
		}
	
		vector<CFileElement *> *vFE = fe->getDirectory();

		// fetch complete size
		for (int h = nMin; h <= nMax; h++)
		{
			nSizeComplete += vFE->at(h)->getSize();
		}

		dCompleteArea	= dWidth * dHeight;
		nSizePivot		= vFE->at(nMiddleID)->getSize();

		// if nMiddleID is first element L1 is empty
		nL1Min = nMin;
		nL1Max = nMiddleID - 1;
		for (int i = nMin; i <= nL1Max; i++) 
			nSizeL1 += vFE->at(i)->getSize();

		
		nSizeRest = nSizeComplete - (nSizePivot + nSizeL1);

		// the square
		dPivotArea   = ((float) nSizePivot / (float) nSizeComplete) * dCompleteArea;
		dPivotWidth	 = sqrt(dPivotArea);
		dPivotHeight = dPivotWidth;

		if (dPivotWidth > dWidth) 
		{
			dPivotWidth  = dWidth;
			dPivotHeight = dPivotArea / dPivotWidth;
		}
		if (dPivotHeight > dHeight)
		{
			dPivotHeight = dHeight;
			dPivotWidth  = dPivotArea / dPivotHeight;
		}

		nPrefSizeL2 = (_int64) (((float) nSizePivot) / dPivotHeight * (dHeight - dPivotHeight));

		int j = nMiddleID + 1;
		while ((j <= nMax) && (nSizeL2 < nPrefSizeL2))
		{
			nSizeL2 += vFE->at(j)->getSize();
			j++;
		}

		// avoid L3 to have just one element
		if (j == nMax) 
		{
			nSizeL2 += vFE->at(j)->getSize();
			j++;
		}

		nL2Min = nMiddleID + 1;
		nL2Max = j - 1;

		// finally fill L3
		nL3Min = nL2Max + 1;
		nL3Max = nMax;

		for (int k = nL3Min; k <= nL3Max; k++)
		{
			nSizeL3 += vFE->at(k)->getSize();
		}


		// all size (areas) are set
		dAreaL1 = ((float) nSizeL1 / (float) nSizeComplete) * dCompleteArea;
		dAreaL2 = ((float) nSizeL2 / (float) nSizeComplete) * dCompleteArea;
		dAreaL3 = ((float) nSizeL3 / (float) nSizeComplete) * dCompleteArea;

		if (nL1Min <= nL1Max)   // L1 - not empty
		{
			dL1Height = dHeight;
			dL1Width  = dAreaL1 / dL1Height;
		}
		else 
		{
			dL1Height = 0.0f;
			dL1Width  = 0.0f;
		}

		if (nL3Min <= nL3Max)	// L3 not empty
		{
			dL3Height = dHeight;
			dL3Width  = dAreaL3 / dL3Height;
		}
		else
		{
			dL3Height = 0.0f;
			dL3Width  = 0.0f;
		}


		if (nL2Min <= nL2Max)
		{
			dL2Width  = dWidth - (dL1Width + dL3Width);
			dL2Height = dAreaL2 / dL2Width;
		}
		else
		{
			dL2Width  = 0.0f;
			dL2Height = 0.0f;
		}

		// this is surely not empty
		dPivotWidth  = dWidth - (dL1Width + dL3Width);
		dPivotHeight = dPivotArea / dPivotWidth;
		
		if (bHorizontal)
		{
			float dPointsPivot[4];
			CFileElement *pPivot = vFE->at(nMiddleID);
			dPointsPivot[0] = dPoints[0] + dL1Width;
			dPointsPivot[1] = dPoints[1] + dL2Height;
			dPointsPivot[2] = dPoints[2] - dL3Width;
			dPointsPivot[3] = dPoints[3];
			setColor(pPivot);
			pPivot->setPoints(dPointsPivot[0], dPointsPivot[1], dPointsPivot[2], dPointsPivot[3]);
			renderPivotBySize(pPivot, 0, pPivot->getDirectory()->size() - 1, dPointsPivot);

			if (dL1Width != 0.0f)
			{
				float dPointsL1[4];
				dPointsL1[0] = dPoints[0];
				dPointsL1[1] = dPoints[1];
				dPointsL1[2] = dPoints[0] + dL1Width;
				dPointsL1[3] = dPoints[1] + dL1Height;
				renderPivotBySize(fe, nL1Min, nL1Max, dPointsL1);
			}

			if (dL2Width != 0.0f)
			{
				float dPointsL2[4];
				dPointsL2[0] = dPoints[0] + dL1Width;
				dPointsL2[1] = dPoints[1];
				dPointsL2[2] = dPoints[2] - dL3Width;
				dPointsL2[3] = dPoints[1] + dL2Height;
				renderPivotBySize(fe, nL2Min, nL2Max, dPointsL2);
			}

			if (dL3Width != 0.0f)
			{
				float dPointsL3[4];
				dPointsL3[0] = dPoints[2] - dL3Width;
				dPointsL3[1] = dPoints[1];
				dPointsL3[2] = dPoints[2];
				dPointsL3[3] = dPoints[3];
				renderPivotBySize(fe, nL3Min, nL3Max, dPointsL3);
			}
		}
		else
		{
			float dPointsPivot[4];
			CFileElement *pPivot = vFE->at(nMiddleID);
			dPointsPivot[0] = dPoints[0];
			dPointsPivot[1] = dPoints[1] + dL1Width;
			dPointsPivot[2] = dPoints[0] + dPivotHeight;
			dPointsPivot[3] = dPoints[3] - dL3Width;
			setColor(pPivot);
			pPivot->setPoints(dPointsPivot[0], dPointsPivot[1], dPointsPivot[2], dPointsPivot[3]);
			renderPivotBySize(pPivot, 0, pPivot->getDirectory()->size() - 1, dPointsPivot);

			if (dL1Width != 0.0f)
			{
				float dPointsL1[4];
				dPointsL1[0] = dPoints[0];
				dPointsL1[1] = dPoints[1];
				dPointsL1[2] = dPoints[2];
				dPointsL1[3] = dPoints[1] + dL1Width;
				renderPivotBySize(fe, nL1Min, nL1Max, dPointsL1);
			}

			if (dL2Width != 0.0f)
			{
				float dPointsL2[4];
				dPointsL2[0] = dPoints[0] + dPivotHeight;
				dPointsL2[1] = dPoints[1] + dL1Width;
				dPointsL2[2] = dPoints[2];
				dPointsL2[3] = dPoints[3] - dL3Width;
				renderPivotBySize(fe, nL2Min, nL2Max, dPointsL2);
			}

			if (dL3Width != 0.0f)
			{
				float dPointsL3[4];
				dPointsL3[0] = dPoints[0];
				dPointsL3[1] = dPoints[3] - dL3Width;
				dPointsL3[2] = dPoints[2];
				dPointsL3[3] = dPoints[3];
				renderPivotBySize(fe, nL3Min, nL3Max, dPointsL3);
			}
		}
/*
		TRACE("FE: %s\n", fe->getName());

		TRACE("Gesamt:  %f\n", dCompleteArea);
		TRACE("L1:		%f\n", dAreaL1);
		TRACE("Pivot:	%f\n", dPivotArea);
		TRACE("L2:		%f\n", dAreaL2);
		TRACE("L3:		%f\n\n", dAreaL3);

		TRACE("Size:   %d\n", nSizeComplete);
		TRACE("L1:     %d\n", nSizeL1);
		TRACE("Pivot:  %d\n", nSizePivot);
		TRACE("L2:     %d\n", nSizeL2);
		TRACE("L3:     %d\n\n\n", nSizeL3);

		TRACE("Borders: %d - %d\n", nMin, nMax);
		TRACE("L1:     %d - %d\n", nL1Min, nL1Max);
		TRACE("Pivot:  %d\n", nMiddleID);
		TRACE("L2:     %d - %d\n", nL2Min, nL2Max);
		TRACE("L3:     %d - %d\n\n\n", nL3Min, nL3Max);
*/
	}
}

//////////////////////////////////////////////////////////////////////

void CRenderer::renderPivotBySize(CFileElement *fe, int nMin, int nMax, float *dPoints)
{
	int nMaxChildID = fe->getLargestChildID(nMin, nMax);

	// if there are childs
	if (nMaxChildID != -1)
	{
		float dWidth, dHeight;

		// sizes
		_int64 nSizeComplete = 0;
		_int64 nSizeRest = 0;
		_int64 nSizeL1 = 0;
		_int64 nSizeL2 = 0;
		_int64 nSizeL3 = 0;
		_int64 nSizePivot = 0;

		_int64 nPrefSizeL2 = 0;

		// list borders
		int nL1Min = 0;
		int nL1Max = 0;
		int nL2Min = 0;
		int nL2Max = 0;
		int nL3Min = 0;
		int nL3Max = 0;

		// pivot square
		float dPivotWidth  = 0.0f;
		float dPivotHeight = 0.0f;
		float dPivotArea   = 0.0f;

		float dCompleteArea = 0.0f;

		float dL1Width  = 0.0f;
		float dL1Height	= 0.0f;
		float dL2Width	= 0.0f;
		float dL2Height = 0.0f;
		float dL3Width  = 0.0f;
		float dL3Height = 0.0f;

		float dAreaL1 = 0.0f;
		float dAreaL2 = 0.0f;
		float dAreaL3 = 0.0f;

		bool bHorizontal = true;

		/*if (dPoints[2] < dPoints[0]) TRACE("error [X]!\n");
		else if (dPoints[3] < dPoints[1]) TRACE("error [Y]!\n");*/

		if ((dPoints[2] - dPoints[0]) >= (dPoints[3] - dPoints[1]))
		{
			dWidth	= fabs(dPoints[2] - dPoints[0]);
			dHeight = fabs(dPoints[3] - dPoints[1]);
		}
		else
		{
			dWidth	= fabs(dPoints[3] - dPoints[1]);
			dHeight = fabs(dPoints[2] - dPoints[0]);
			bHorizontal = false;
		}
	
		vector<CFileElement *> *vFE = fe->getDirectory();

		// fetch complete size
		for (int h = nMin; h <= nMax; h++)
		{
			nSizeComplete += vFE->at(h)->getSize();
		}

		dCompleteArea	= dWidth * dHeight;
		nSizePivot		= vFE->at(nMaxChildID)->getSize();

		// if nMaxChildID is first element L1 is empty
		nL1Min = nMin;
		nL1Max = nMaxChildID - 1;
		for (int i = nMin; i <= nL1Max; i++) 
			nSizeL1 += vFE->at(i)->getSize();

		
		nSizeRest = nSizeComplete - (nSizePivot + nSizeL1);

		// the square
		dPivotArea   = ((float) nSizePivot / (float) nSizeComplete) * dCompleteArea;
		dPivotWidth	 = sqrt(dPivotArea);
		dPivotHeight = dPivotWidth;

		if (dPivotWidth > dWidth) 
		{
			dPivotWidth  = dWidth;
			dPivotHeight = dPivotArea / dPivotWidth;
		}
		if (dPivotHeight > dHeight)
		{
			dPivotHeight = dHeight;
			dPivotWidth  = dPivotArea / dPivotHeight;
		}

		nPrefSizeL2 = (_int64) (((float) nSizePivot) / dPivotHeight * (dHeight - dPivotHeight));

		int j = nMaxChildID + 1;
		while ((j <= nMax) && (nSizeL2 < nPrefSizeL2))
		{
			nSizeL2 += vFE->at(j)->getSize();
			j++;
		}

		// avoid L3 to have just one element
		if (j == nMax) 
		{
			nSizeL2 += vFE->at(j)->getSize();
			j++;
		}

		nL2Min = nMaxChildID + 1;
		nL2Max = j - 1;

		// finally fill L3
		nL3Min = nL2Max + 1;
		nL3Max = nMax;

		for (int k = nL3Min; k <= nL3Max; k++)
		{
			nSizeL3 += vFE->at(k)->getSize();
		}


		// all size (areas) are set
		dAreaL1 = ((float) nSizeL1 / (float) nSizeComplete) * dCompleteArea;
		dAreaL2 = ((float) nSizeL2 / (float) nSizeComplete) * dCompleteArea;
		dAreaL3 = ((float) nSizeL3 / (float) nSizeComplete) * dCompleteArea;

		if (nL1Min <= nL1Max)   // L1 - not empty
		{
			dL1Height = dHeight;
			dL1Width  = dAreaL1 / dL1Height;
		}
		else 
		{
			dL1Height = 0.0f;
			dL1Width  = 0.0f;
		}

		if (nL3Min <= nL3Max)	// L3 not empty
		{
			dL3Height = dHeight;
			dL3Width  = dAreaL3 / dL3Height;
		}
		else
		{
			dL3Height = 0.0f;
			dL3Width  = 0.0f;
		}


		if (nL2Min <= nL2Max)
		{
			dL2Width  = dWidth - (dL1Width + dL3Width);
			dL2Height = dAreaL2 / dL2Width;
		}
		else
		{
			dL2Width  = 0.0f;
			dL2Height = 0.0f;
		}

		// this is surely not empty
		dPivotWidth  = dWidth - (dL1Width + dL3Width);
		dPivotHeight = dPivotArea / dPivotWidth;
		
		if (bHorizontal)
		{
			float dPointsPivot[4];
			CFileElement *pPivot = vFE->at(nMaxChildID);
			dPointsPivot[0] = dPoints[0] + dL1Width;
			dPointsPivot[1] = dPoints[1] + dL2Height;
			dPointsPivot[2] = dPoints[2] - dL3Width;
			dPointsPivot[3] = dPoints[3];
			setColor(pPivot);
			pPivot->setPoints(dPointsPivot[0], dPointsPivot[1], dPointsPivot[2], dPointsPivot[3]);
			renderPivotBySize(pPivot, 0, pPivot->getDirectory()->size() - 1, dPointsPivot);

			if (dL1Width != 0.0f)
			{
				float dPointsL1[4];
				dPointsL1[0] = dPoints[0];
				dPointsL1[1] = dPoints[1];
				dPointsL1[2] = dPoints[0] + dL1Width;
				dPointsL1[3] = dPoints[1] + dL1Height;
				renderPivotBySize(fe, nL1Min, nL1Max, dPointsL1);
			}

			if (dL2Width != 0.0f)
			{
				float dPointsL2[4];
				dPointsL2[0] = dPoints[0] + dL1Width;
				dPointsL2[1] = dPoints[1];
				dPointsL2[2] = dPoints[2] - dL3Width;
				dPointsL2[3] = dPoints[1] + dL2Height;
				renderPivotBySize(fe, nL2Min, nL2Max, dPointsL2);
			}

			if (dL3Width != 0.0f)
			{
				float dPointsL3[4];
				dPointsL3[0] = dPoints[2] - dL3Width;
				dPointsL3[1] = dPoints[1];
				dPointsL3[2] = dPoints[2];
				dPointsL3[3] = dPoints[3];
				renderPivotBySize(fe, nL3Min, nL3Max, dPointsL3);
			}
		}
		else
		{
			float dPointsPivot[4];
			CFileElement *pPivot = vFE->at(nMaxChildID);
			dPointsPivot[0] = dPoints[0];
			dPointsPivot[1] = dPoints[1] + dL1Width;
			dPointsPivot[2] = dPoints[0] + dPivotHeight;
			dPointsPivot[3] = dPoints[3] - dL3Width;
			setColor(pPivot);
			pPivot->setPoints(dPointsPivot[0], dPointsPivot[1], dPointsPivot[2], dPointsPivot[3]);
			renderPivotBySize(pPivot, 0, pPivot->getDirectory()->size() - 1, dPointsPivot);

			if (dL1Width != 0.0f)
			{
				float dPointsL1[4];
				dPointsL1[0] = dPoints[0];
				dPointsL1[1] = dPoints[1];
				dPointsL1[2] = dPoints[2];
				dPointsL1[3] = dPoints[1] + dL1Width;
				renderPivotBySize(fe, nL1Min, nL1Max, dPointsL1);
			}

			if (dL2Width != 0.0f)
			{
				float dPointsL2[4];
				dPointsL2[0] = dPoints[0] + dPivotHeight;
				dPointsL2[1] = dPoints[1] + dL1Width;
				dPointsL2[2] = dPoints[2];
				dPointsL2[3] = dPoints[3] - dL3Width;
				renderPivotBySize(fe, nL2Min, nL2Max, dPointsL2);
			}

			if (dL3Width != 0.0f)
			{
				float dPointsL3[4];
				dPointsL3[0] = dPoints[0];
				dPointsL3[1] = dPoints[3] - dL3Width;
				dPointsL3[2] = dPoints[2];
				dPointsL3[3] = dPoints[3];
				renderPivotBySize(fe, nL3Min, nL3Max, dPointsL3);
			}
		}
/*
		TRACE("FE: %s\n", fe->getName());

		TRACE("Gesamt:  %f\n", dCompleteArea);
		TRACE("L1:		%f\n", dAreaL1);
		TRACE("Pivot:	%f\n", dPivotArea);
		TRACE("L2:		%f\n", dAreaL2);
		TRACE("L3:		%f\n\n", dAreaL3);

		TRACE("Size:   %d\n", nSizeComplete);
		TRACE("L1:     %d\n", nSizeL1);
		TRACE("Pivot:  %d\n", nSizePivot);
		TRACE("L2:     %d\n", nSizeL2);
		TRACE("L3:     %d\n\n\n", nSizeL3);

		TRACE("Borders: %d - %d\n", nMin, nMax);
		TRACE("L1:     %d - %d\n", nL1Min, nL1Max);
		TRACE("Pivot:  %d\n", nMaxChildID);
		TRACE("L2:     %d - %d\n", nL2Min, nL2Max);
		TRACE("L3:     %d - %d\n\n\n", nL3Min, nL3Max);
*/
	}
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
