// Renderer.h: Schnittstelle fr die Klasse CRenderer.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_RENDERER_H__5931280C_49EA_46F4_A5F9_CC74639B51F7__INCLUDED_)
#define AFX_RENDERER_H__5931280C_49EA_46F4_A5F9_CC74639B51F7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "FileElement.h"
#include <vector>
using namespace std;

class CRenderer
{
public:
	CRenderer();
	virtual ~CRenderer();

	enum RENDERTYPE		{ SLICE_AND_DICE, SQUARIFIED, PIVOT_BY_MIDDLE, PIVOT_BY_SIZE };
	enum COLATTRIBUTE	{ CREATION, MODIFIED, RANDOM, SIZE };
	
	// getters
	inline bool IsDirty()					{	return m_bDirty;		}
	inline void ClearDirtyBit()				{	m_bDirty = false;		}
	inline RENDERTYPE getRenderType()		{	return m_eType;			}	
	inline CFileElement *getFileElement()	{	return m_FileElement;	}

	// setters
	void setRenderType(RENDERTYPE rt);
	void setColorAttribute(COLATTRIBUTE att);
	void setFileElement(CFileElement *fe);
	void setMinColor(float dRed, float dGreen, float dBlue);
	void setMaxColor(float dRed, float dGreen, float dBlue);

	// do the rendering (precalculation)
	void render();

protected:
	bool m_bDirty;					// indicates if something has changed 
	CFileElement *m_FileElement;	// pointer to the Tree-Object
	RENDERTYPE m_eType;				// the rendertype
	COLATTRIBUTE m_eColAttr;		// the coloring attribute
	_int64 m_nMaxSize;				// size of all the files

	float m_dMinCol[3];				// minimum color
	float m_dMaxCol[3];				// maximum color

	float m_dDiffCol[3];			// difference between m_dMaxCol and m_dMinCol

	float m_dRange;					// for color calculations
	float m_dMinValue;

	// get a color (dummy-function)
	void setColor(CFileElement *fe);

	// calculate the difference between max and min color
	void CalcColorDiff();

	// protected rendering algorithms
	void renderSliceAndDice(CFileElement *fe, bool bHorizontal);
	void renderPivotByMiddle(CFileElement *fe, int nMin, int nMax, float *dPoints);
	void renderPivotBySize(CFileElement *fe, int nMin, int nMax, float *dPoints);

	bool m_bHasRandomColor;
};

#endif // !defined(AFX_RENDERER_H__5931280C_49EA_46F4_A5F9_CC74639B51F7__INCLUDED_)
